@@ -659,9 +659,6 @@ gst_omx_video_dec_allocate_output_buffers (GstOMXVideoDec * self)
       }
       gl_mem = (GstGLMemoryEGL *) mem;
       buffers = g_list_append (buffers, buffer);
-      /* FIXME: replace with the affine transformation meta */
-      gl_mem->image->orientation =
-          GST_VIDEO_GL_TEXTURE_ORIENTATION_X_NORMAL_Y_FLIP;
       images = g_list_append (images, gst_gl_memory_egl_get_image (gl_mem));
       if (egl_display == EGL_NO_DISPLAY)
         egl_display = gst_gl_memory_egl_get_display (gl_mem);
@@ -971,7 +968,7 @@ gst_omx_video_dec_reconfigure_output_port (GstOMXVideoDec * self)
         /* fallback: try to use EGLImage even if it is not in the caps feature */
         if (!gst_video_decoder_negotiate (GST_VIDEO_DECODER (self))) {
           gst_video_codec_state_unref (state);
-          GST_ERROR_OBJECT (self, "Failed to negotiate RGBA for EGLImage");
+          GST_DEBUG_OBJECT (self, "Failed to negotiate RGBA for EGLImage");
           GST_VIDEO_DECODER_STREAM_UNLOCK (self);
           goto no_egl;
         }
@@ -1189,11 +1186,11 @@ gst_omx_video_dec_clean_older_frames (GstOMXVideoDec * self,
       GstVideoCodecFrame *tmp = l->data;
 
       if (tmp->pts < timestamp) {
-        gst_video_decoder_release_frame (GST_VIDEO_DECODER (self), tmp);
         GST_LOG_OBJECT (self,
             "discarding ghost frame %p (#%d) PTS:%" GST_TIME_FORMAT " DTS:%"
             GST_TIME_FORMAT, tmp, tmp->system_frame_number,
             GST_TIME_ARGS (tmp->pts), GST_TIME_ARGS (tmp->dts));
+        gst_video_decoder_release_frame (GST_VIDEO_DECODER (self), tmp);
       } else {
         gst_video_codec_frame_unref (tmp);
       }
@@ -1205,11 +1202,11 @@ gst_omx_video_dec_clean_older_frames (GstOMXVideoDec * self,
       GstVideoCodecFrame *tmp = l->data;
 
       if (!GST_CLOCK_TIME_IS_VALID (tmp->pts)) {
-        gst_video_decoder_release_frame (GST_VIDEO_DECODER (self), tmp);
         GST_LOG_OBJECT (self,
             "discarding frame %p (#%d) with invalid PTS:%" GST_TIME_FORMAT
             " DTS:%" GST_TIME_FORMAT, tmp, tmp->system_frame_number,
             GST_TIME_ARGS (tmp->pts), GST_TIME_ARGS (tmp->dts));
+        gst_video_decoder_release_frame (GST_VIDEO_DECODER (self), tmp);
       } else {
         gst_video_codec_frame_unref (tmp);
       }
