@@ -1025,6 +1025,12 @@ gst_omx_video_enc_set_format (GstVideoEncoder * encoder,
       case GST_VIDEO_FORMAT_NV12:
         port_def.format.video.eColorFormat = OMX_COLOR_FormatYUV420SemiPlanar;
         break;
+      case GST_VIDEO_FORMAT_ABGR:
+        port_def.format.video.eColorFormat = OMX_COLOR_Format32bitARGB8888;
+        break;
+      case GST_VIDEO_FORMAT_ARGB:
+        port_def.format.video.eColorFormat = OMX_COLOR_Format32bitBGRA8888;
+        break;
       default:
         GST_ERROR_OBJECT (self, "Unsupported format %s",
             gst_video_format_to_string (info->finfo->format));
@@ -1047,13 +1053,16 @@ gst_omx_video_enc_set_format (GstVideoEncoder * encoder,
   port_def.format.video.nFrameWidth = info->width;
   if (port_def.nBufferAlignment)
     port_def.format.video.nStride =
-        (info->width + port_def.nBufferAlignment - 1) &
-        (~(port_def.nBufferAlignment - 1));
+        GST_ROUND_UP_N (info->width, port_def.nBufferAlignment);
   else
     port_def.format.video.nStride = GST_ROUND_UP_4 (info->width);       /* safe (?) default */
 
   port_def.format.video.nFrameHeight = info->height;
-  port_def.format.video.nSliceHeight = info->height;
+
+  if (klass->cdata.hacks & GST_OMX_HACK_HEIGHT_MULTIPLE_16)
+    port_def.format.video.nSliceHeight = GST_ROUND_UP_16 (info->height);
+  else
+    port_def.format.video.nSliceHeight = info->height;
 
   switch (port_def.format.video.eColorFormat) {
     case OMX_COLOR_FormatYUV420Planar:
@@ -1064,6 +1073,7 @@ gst_omx_video_enc_set_format (GstVideoEncoder * encoder,
           ((port_def.format.video.nFrameHeight + 1) / 2));
       break;
 
+    case OMX_COLOR_FormatYUV420PackedSemiPlanar:
     case OMX_COLOR_FormatYUV420SemiPlanar:
       port_def.nBufferSize =
           (port_def.format.video.nStride * port_def.format.video.nFrameHeight) +
@@ -1072,6 +1082,8 @@ gst_omx_video_enc_set_format (GstVideoEncoder * encoder,
       break;
 
     default:
+      GST_ERROR_OBJECT (self, "Unsupported port format %x",
+          port_def.format.video.eColorFormat);
       g_assert_not_reached ();
   }
 
@@ -1300,6 +1312,9 @@ gst_omx_video_enc_fill_buffer (GstOMXVideoEnc * self, GstBuffer * inbuf,
       outbuf->omx_buf->nAllocLen - outbuf->omx_buf->nOffset) {
     outbuf->omx_buf->nFilledLen = gst_buffer_get_size (inbuf);
 
+    GST_LOG_OBJECT (self, "Matched strides - direct copy %u bytes",
+        (guint) outbuf->omx_buf->nFilledLen);
+
     gst_buffer_extract (inbuf, 0,
         outbuf->omx_buf->pBuffer + outbuf->omx_buf->nOffset,
         outbuf->omx_buf->nFilledLen);
@@ -1308,10 +1323,11 @@ gst_omx_video_enc_fill_buffer (GstOMXVideoEnc * self, GstBuffer * inbuf,
   }
 
   /* Different strides */
+  GST_LOG_OBJECT (self, "Mismatched strides - copying line-by-line");
 
   switch (info->finfo->format) {
     case GST_VIDEO_FORMAT_I420:{
-      gint i, j, height, width;
+      gint i, j, height, width, dest_height;
       guint8 *src, *dest;
       gint src_stride, dest_stride;
 
@@ -1326,20 +1342,17 @@ gst_omx_video_enc_fill_buffer (GstOMXVideoEnc * self, GstBuffer * inbuf,
       for (i = 0; i < 3; i++) {
         if (i == 0) {
           dest_stride = port_def->format.video.nStride;
-          src_stride = GST_VIDEO_FRAME_COMP_STRIDE (&frame, 0);
-
-          /* XXX: Try this if no stride was set */
-          if (dest_stride == 0)
-            dest_stride = src_stride;
+          dest_height = port_def->format.video.nSliceHeight;
         } else {
           dest_stride = port_def->format.video.nStride / 2;
-          src_stride = GST_VIDEO_FRAME_COMP_STRIDE (&frame, 1);
-
-          /* XXX: Try this if no stride was set */
-          if (dest_stride == 0)
-            dest_stride = src_stride;
+          dest_height = port_def->format.video.nSliceHeight / 2;
         }
 
+        src_stride = GST_VIDEO_FRAME_COMP_STRIDE (&frame, i);
+        /* XXX: Try this if no stride was set */
+        if (dest_stride == 0)
+          dest_stride = src_stride;
+
         dest = outbuf->omx_buf->pBuffer + outbuf->omx_buf->nOffset;
         if (i > 0)
           dest +=
@@ -1368,13 +1381,19 @@ gst_omx_video_enc_fill_buffer (GstOMXVideoEnc * self, GstBuffer * inbuf,
           src += src_stride;
           dest += dest_stride;
         }
+        for (; j < dest_height; j++) {
+          memset (dest, 0, dest_stride);
+          outbuf->omx_buf->nFilledLen += dest_stride;
+          src += src_stride;
+          dest += dest_stride;
+        }
       }
       gst_video_frame_unmap (&frame);
       ret = TRUE;
       break;
     }
     case GST_VIDEO_FORMAT_NV12:{
-      gint i, j, height, width;
+      gint i, j, height, width, dest_height;
       guint8 *src, *dest;
       gint src_stride, dest_stride;
 
@@ -1385,22 +1404,13 @@ gst_omx_video_enc_fill_buffer (GstOMXVideoEnc * self, GstBuffer * inbuf,
         ret = FALSE;
         break;
       }
+      dest_stride = port_def->format.video.nStride;
 
       for (i = 0; i < 2; i++) {
-        if (i == 0) {
-          dest_stride = port_def->format.video.nStride;
-          src_stride = GST_VIDEO_FRAME_COMP_STRIDE (&frame, 0);
-          /* XXX: Try this if no stride was set */
-          if (dest_stride == 0)
-            dest_stride = src_stride;
-        } else {
-          dest_stride = port_def->format.video.nStride;
-          src_stride = GST_VIDEO_FRAME_COMP_STRIDE (&frame, 1);
-
-          /* XXX: Try this if no stride was set */
-          if (dest_stride == 0)
-            dest_stride = src_stride;
-        }
+        src_stride = GST_VIDEO_FRAME_COMP_STRIDE (&frame, i);
+        /* XXX: Try this if no stride was set */
+        if (dest_stride == 0)
+          dest_stride = src_stride;
 
         dest = outbuf->omx_buf->pBuffer + outbuf->omx_buf->nOffset;
         if (i == 1)
@@ -1408,6 +1418,10 @@ gst_omx_video_enc_fill_buffer (GstOMXVideoEnc * self, GstBuffer * inbuf,
               port_def->format.video.nSliceHeight *
               port_def->format.video.nStride;
 
+        if (i == 0)
+          dest_height = port_def->format.video.nSliceHeight;
+        else
+          dest_height = port_def->format.video.nSliceHeight / 2;
         src = GST_VIDEO_FRAME_COMP_DATA (&frame, i);
         height = GST_VIDEO_FRAME_COMP_HEIGHT (&frame, i);
         width = GST_VIDEO_FRAME_COMP_WIDTH (&frame, i) * (i == 0 ? 1 : 2);
@@ -1426,7 +1440,12 @@ gst_omx_video_enc_fill_buffer (GstOMXVideoEnc * self, GstBuffer * inbuf,
           src += src_stride;
           dest += dest_stride;
         }
-
+        for (; j < dest_height; j++) {
+          memset (dest, 0, dest_stride);
+          outbuf->omx_buf->nFilledLen += dest_stride;
+          src += src_stride;
+          dest += dest_stride;
+        }
       }
       gst_video_frame_unmap (&frame);
       ret = TRUE;
@@ -1732,6 +1751,7 @@ gst_omx_video_enc_drain (GstOMXVideoEnc * self)
   if (err != OMX_ErrorNone) {
     GST_ERROR_OBJECT (self, "Failed to drain component: %s (0x%08x)",
         gst_omx_error_to_string (err), err);
+    g_mutex_unlock (&self->drain_lock);
     GST_VIDEO_ENCODER_STREAM_LOCK (self);
     return GST_FLOW_ERROR;
   }
@@ -1757,12 +1777,40 @@ gst_omx_video_enc_propose_allocation (GstVideoEncoder * encoder,
       (gst_omx_video_enc_parent_class)->propose_allocation (encoder, query);
 }
 
+static GList *
+filter_supported_formats (GList * negotiation_map)
+{
+  GList *cur;
+
+  for (cur = negotiation_map; cur != NULL;) {
+    GstOMXVideoNegotiationMap *nmap = (GstOMXVideoNegotiationMap *) (cur->data);
+    GList *next;
+
+    switch (nmap->format) {
+      case GST_VIDEO_FORMAT_I420:
+      case GST_VIDEO_FORMAT_NV12:
+        //case GST_VIDEO_FORMAT_ABGR:
+        //case GST_VIDEO_FORMAT_ARGB:
+        cur = g_list_next (cur);
+        continue;
+      default:
+        gst_omx_video_negotiation_map_free (nmap);
+        next = g_list_next (cur);
+        negotiation_map = g_list_delete_link (negotiation_map, cur);
+        cur = next;
+    }
+  }
+
+  return negotiation_map;
+}
+
 static GstCaps *
 gst_omx_video_enc_getcaps (GstVideoEncoder * encoder, GstCaps * filter)
 {
   GstOMXVideoEnc *self = GST_OMX_VIDEO_ENC (encoder);
   GList *negotiation_map = NULL;
   GstCaps *comp_supported_caps;
+  GstCaps *ret;
 
   if (!self->enc)
     return gst_video_encoder_proxy_getcaps (encoder, NULL, filter);
@@ -1770,18 +1818,22 @@ gst_omx_video_enc_getcaps (GstVideoEncoder * encoder, GstCaps * filter)
   negotiation_map =
       gst_omx_video_get_supported_colorformats (self->enc_in_port,
       self->input_state);
+  negotiation_map = filter_supported_formats (negotiation_map);
+
   comp_supported_caps = gst_omx_video_get_caps_for_map (negotiation_map);
   g_list_free_full (negotiation_map,
       (GDestroyNotify) gst_omx_video_negotiation_map_free);
 
   if (!gst_caps_is_empty (comp_supported_caps)) {
-    GstCaps *ret =
+    ret =
         gst_video_encoder_proxy_getcaps (encoder, comp_supported_caps, filter);
-
     gst_caps_unref (comp_supported_caps);
-    return ret;
   } else {
     gst_caps_unref (comp_supported_caps);
-    return gst_video_encoder_proxy_getcaps (encoder, NULL, filter);
+    ret = gst_video_encoder_proxy_getcaps (encoder, NULL, filter);
   }
+
+  GST_LOG_OBJECT (encoder, "Supported caps %" GST_PTR_FORMAT, ret);
+
+  return ret;
 }
